<?php
/**
 * Plugin Name: Bizuno Accounting/ERP/CRM
 * Plugin URI:  https://www.phreesoft.com
 * Description: Bizuno is a powerful ERP/Accounting application adapted as a plugin for WordPress. Bizuno creates a portal running within the WordPress administrator. Once activated, click on the Bizuno menu item to complete the installation and access your Bizuno business.
 * Version:     6.5.4
 * Author:      PhreeSoft, Inc.
 * Author URI:  http://www.PhreeSoft.com
 * Text Domain: bizuno
 * License:     GPL3
 * License URI: https://www.gnu.org/licenses/gpl.html
 * Domain Path: /locale
 */

if (!defined( 'ABSPATH' )) { die( 'No script kiddies please!' ); }

if (!defined('MODULE_BIZUNO_VERSION')) { define('MODULE_BIZUNO_VERSION', '6.5.4'); }

class bizuno_accounting
{
    public function __construct()
    {
        // Actions
        add_action ( 'init',                              [ $this, 'init_bizuno'] );
        add_action ( 'admin_init',                        [ $this, 'admin_init_bizuno' ] );
        add_action ( 'admin_menu',                        [ $this, 'admin_menu_bizuno' ] );
        add_action ( 'wp_before_admin_bar_render',        [ $this, 'bizuno_admin_menu_mods'] );
        add_action ( 'phpmailer_init',                    [ $this, 'bizuno_phpmailer_init' ], 10, 1 );
        add_action ( 'template_redirect',                 [ $this, 'bizunoPageRedirect' ] );
//      add_action ( 'wp_ajax_bizoffice_ajax',            [ $this, 'bizOfficeAjax'] );
//      add_action ( 'wp_ajax_nopriv_bizoffice_ajax',     [ $this, 'bizOfficeAjax'] );
        add_action ( 'wp_ajax_bizuno_ajax',               [ $this, 'bizunoAjax' ] );
        add_action ( 'wp_ajax_nopriv_bizuno_ajax',        [ $this, 'bizunoAjax' ] );
        add_action ( 'wp_ajax_bizuno_ajax_fs',            [ $this, 'bizunoAjaxFs' ] );
        add_action ( 'wp_ajax_nopriv_bizuno_ajax_fs',     [ $this, 'bizunoAjaxFs' ] );
        add_action ( 'wp_ajax_bizuno_api',                [ $this, 'bizuno_api' ] );
        add_action ( 'wp_ajax_nopriv_bizuno_api',         [ $this, 'bizuno_api' ] );
        add_action ( 'wp_ajax_bizuno_api_order_download', [ $this, 'bizuno_api_manual_download' ], 10 );
        add_action ( 'edit_user_profile',                 [ $this, 'bizunoUserEdit'] ); // maybe change to show_user_profile
        add_action ( 'show_user_profile',                 [ $this, 'bizunoUserEdit'] );
        add_action ( 'personal_options_update',           [ $this, 'bizunoUserSave'] );
        add_action ( 'edit_user_profile_update',          [ $this, 'bizunoUserSave'] );
        add_action ( 'wp_logout',                         [ $this, 'bizuno_user_logout' ] );
        add_action ( 'bizuno_daily_event',                [ $this, 'daily_cron' ] );
        add_action ( 'admin_notices',                     [ $this, 'bizuno_api_display_notices' ], 12 );
        add_action ( 'woocommerce_thankyou',              [ $this, 'bizuno_api_post_payment' ], 10, 1 );
        add_action ( 'woocommerce_order_action_bizuno_export_action',[ $this, 'bizuno_api_process_order_meta_box_action' ] );
        add_action ( 'manage_shop_order_posts_custom_column',        [ $this, 'bizuno_api_order_column_content' ] );
        add_action ( 'woocommerce_admin_order_preview_end',          [ $this, 'bizuno_api_order_preview_action' ] );
        add_action ( 'shutdown',                          [$this, 'bizuno_debug_log_write'] );
        // Filters
        add_filter ( 'woocommerce_get_settings_advanced', [ $this, 'bizuno_api_all_setting' ], 10, 2 );
        add_filter ( 'wc_order_statuses',                 [ $this, 'add_shipped_to_order_statuses' ] );
        add_filter ( 'woocommerce_get_sections_advanced', [ $this, 'bizuno_api_add_section' ] );
        add_filter ( 'manage_edit-shop_order_columns',    [ $this, 'bizuno_api_order_column_header' ], 20 );
        add_filter ( 'woocommerce_admin_order_preview_get_order_details',[ $this, 'bizuno_api_order_preview_filter' ], 10, 2 );
        add_filter ( 'woocommerce_order_actions',         [ $this, 'bizuno_api_add_order_meta_box_filter' ] );
        // Install/Uninstall hooks
        register_activation_hook( __FILE__ ,              [ $this, 'activate'] );
        register_deactivation_hook( __FILE__ ,            [ $this, 'deactivate'] );
        register_uninstall_hook( __FILE__,                'bizuno_uninstall' ); // do not put inside of class
    }

    // Shows the admin menu if the user is logged in and has permission to reach Bizuno Accounting
    public function init_bizuno()
    {
        require_once('bizunoCFG.php');
        if ( $this->validateUser() ) { add_filter('show_admin_bar', '__return_true', 1000); }
        /********* API *********/
        register_post_status( 'wc-shipped', ['label'=>'Shipped','public'=>true,'exclude_from_search'=>false,'show_in_admin_all_list'=>true,'show_in_admin_status_list'=>true,
            'label_count'=>_n_noop( 'Shipped <span class="count">(%s)</span>', 'Shipped <span class="count">(%s)</span>' )] );
    }

    public function admin_init_bizuno()
    {
        add_option( 'bizuno_pro_key', '');
        register_setting( 'bizuno_options_group', 'bizuno_pro_key', 'bizuno_callback' );
    }

    public function admin_menu_bizuno()
    {
        add_menu_page('Bizuno Accounting', 'Bizuno', 'manage_options', 'bizuno', 'bizuno_html', plugins_url( 'view/images/icon_16.png', __FILE__ ), 90);
//      add_options_page('Bizuno Settings','Bizuno', 'manage_options', 'bizuno-accounting', 'bizuno_options_page');
    }

    /**
     * Inserts the Bizuno Accounting menu items to the admin toolbar before the logout selection
     * NOTE: Show admin toolbar needs to be enabled for non-admins to see the menu
     */
    public function bizuno_admin_menu_mods()
    {
        global $wp_admin_bar;
        if ( !$this->validateUser() ) { return; }
        if (empty(get_page_by_path('bizuno'))) { return; }
        $logout = $wp_admin_bar->get_node('logout');
        $wp_admin_bar->remove_node( 'logout' );
        $wp_admin_bar->add_node( ['id'=>'tb-admin-bizBooks', 'title'=>'Bizuno Accounting','href'=>BIZUNO_HOME,   'parent'=>'user-actions', 'meta'=>['target'=>'_blank']] );
//      $wp_admin_bar->add_node( ['id'=>'tb-admin-bizOffice','title'=>'Bizuno Office',    'href'=>BIZOFFICE_HOME,'parent'=>'user-actions', 'meta'=>['target'=>'_blank']] );
        $wp_admin_bar->add_node( $logout );
    }

    public function bizuno_phpmailer_init( $phpmailer )
    {
        if ( get_post_field( 'post_name' ) == 'bizuno' ) { $phpmailer->IsHTML( true ); } // set email format to HTML
    }

    public function bizunoCtlr()
    {
        if ( !$this->validateUser() ) { return; }
        require_once ('controllers/portal/controller.php');
        return new \bizuno\portalCtl();
    }

    // Controller for bizOffice Storage home page
    public function bizOfficeCtlr()
    {
//      if (empty(get_user_meta( get_current_user_id(), 'bizoffice_enable', true))) { return; }
//      require_once ('bizOffice.php');
//      $ctl   = new \bizuno\bizOfficeCtl();
//      $ctl->compose();
//      new \bizuno\view($ctl->layout);
//      exit();
    }

    public function bizunoPageRedirect() {
        global $post;
        if ( is_user_logged_in() && !empty($post->post_name) && 'bizuno'==$post->post_name) {
            $ctl = $this->bizunoCtlr();
            $ctl->compose();
            new \bizuno\view($ctl->layout);
            exit();
        }
//      if ( !empty($post->post_name) && 'bizuno-office' == $post->post_name) { $this->bizOfficeCtlr(); }
    }

    // Launch method for all bizOffice AJAX transactions
    public function bizOfficeAjax()
    {
//      if ( is_user_logged_in() ) { $this->bizOfficeCtlr(); }
    }

    // Launch method for all Bizuno AJAX transactions
    public function bizunoAjax()
    {
        if ( is_user_logged_in() ) {
            $ctl = $this->bizunoCtlr();
            $ctl->compose();
            new \bizuno\view($ctl->layout);
            exit();
        }
    }

    // for the file system restricted to wp-content/uploads/bizuno and below ONLY!
    public function bizunoAjaxFs()
    {
        require_once ("bizunoFS.php");
        wp_die();
    }

    // Used to upload to WooCommerce but probably will be done directly
    public function bizuno_api() {
        require_once( __FILE__ . 'controllers/api/main.php');
        $ctl = new \bizuno\apiMain();
        $ctl->processRcv();
        die (json_encode($ctl->arrMsg));
    }

    public function bizuno_api_manual_download($order_id=0) {
        $this->bizuno_api_order_download($order_id, true);
        wp_redirect( esc_url(admin_url( 'edit.php?post_type=shop_order') ) );
        exit;
    }

    public function bizunoUserEdit( $user )
    {
        if (!is_admin()) { return; }
        require_once (BIZBOOKS_ROOT ."controllers/portal/functions.php");
        $html  = '<h3 class="heading">Bizuno Accounting User Permissions</h3>';
        $html .= '<table class="form-table">';
        $html .= '  <tr><th><label for="bizbooks_contact_id">Bizuno Contact ID (for price overrides)</label></th>';
        $html .= '      <td><input type="text" name="bizbooks_contact_id" id="bizbooks_contact_id" value="'.get_user_meta( $user->ID, 'bizbooks_contact_id', true).'"/></td></tr>';
        $html .= '  <tr><th><label for="bizbooks_enable_po">Bizuno Allow PO Payment (requires PhreeSoft plugin)</label></th>';
        $html .= '      <td><input type="text" name="bizbooks_enable_po" id="bizbooks_enable_po" value="'.get_user_meta( $user->ID, 'bizbooks_enable_po', true).'"/></td></tr>';
        foreach ($GLOBALS['bizPortal'] as $biz) {
            $enabled = get_user_meta( $user->ID, 'bizbooks_enable_'.$biz['id'], true);
            $checked = !empty($enabled) ? ' checked' : '';
            $roleID  = get_user_meta( $user->ID, 'bizbooks_role_'  .$biz['id'], true);
            $dbTmp   = new wpdb($biz['user'], $biz['pass'], $biz['name'], $biz['host']);
            if (empty($dbTmp)) { continue; }
            if ( $dbTmp->get_var( "SHOW TABLES LIKE '{$biz['prefix']}configuration'" ) == $biz['prefix'].'configuration' ) {
                $values  = $dbTmp->get_row("SELECT * FROM `{$biz['prefix']}configuration` WHERE config_key='bizuno'");
                $setting = $values ? json_decode($values->config_value, true) : [];
                $title   = !empty($setting['settings']['company']['primary_name']) ? $setting['settings']['company']['primary_name'] : $biz['title'];
                $roles   = $dbTmp->get_results("SELECT id, title FROM `{$biz['prefix']}roles` WHERE inactive='0'");
            } else {
                $title   = $biz['title'];
                $roles   = [(object)['id'=>1, 'title'=>'Admin']];
            }
            $html .= '<tr><th><label for="contact">Allow access to: '.$title.'</label></th>';
            $html .= '  <td><input type="checkbox" class="input-checkbox form-control" id="bizbooks_enable_'.$biz['id'].'" name="bizbooks_enable_'.$biz['id'].'"'.$checked.' /></td></tr>';
            // get the role pull down for each business
            if (!empty($roles)) { // error of some kind, maybe not initialized, or no entries which is bad
                $html .= '<tr><th>Role:</th><td>';
                $html .= '<select name="bizbooks_role_'.$biz['id'].'" id="bizbooks_role_'.$biz['id'].'">';
                foreach ($roles as $role) { $html .= '<option value="'.$role->id.'"'.($roleID==$role->id?' selected':'').'>'.$role->title.'</option>'; }
                $html .= '</select>';
            } else { // might not be initialized, set to 1 which is the default admin
                $html .= '<input type="hidden" name="bizbooks_role_'.$biz['id'].'" id="bizbooks_role_'.$biz['id'].'" value="1" />';
            }
            $html .= '</td></tr>';
        }
        $html .= '</table>';
        echo $html;
    }

    public function bizunoUserSave( $user_id )
    {
        if (!is_admin()) { return; }
        $this->bizunoCtlr(); // load Bizuno functions
        $user = get_userdata( $user_id );
        if (empty($user)) { return; }
        update_user_meta( $user_id, 'bizbooks_contact_id', $_POST['bizbooks_contact_id'] );
        update_user_meta( $user_id, 'bizbooks_enable_po',  $_POST['bizbooks_enable_po'] );
        foreach ($GLOBALS['bizPortal'] as $biz) {
            $enabled = !empty( $_POST['bizbooks_enable_'.$biz['id']] ) ? true : false;
            if ($enabled) { \bizuno\bizSetUser($user, $biz['id']); }
            else          { \bizuno\bizDelUser($user, $biz['id']); }
        }
    }

    public function bizuno_user_logout()
    {
        $redirect_url = site_url();
        wp_safe_redirect( $redirect_url );
        exit;
    }

    // Adds Bizuno meta to order (last call before completed). May also download order if set to do so
    public function bizuno_api_post_payment($order_id) {
        if (!empty(get_post_meta($order_id, 'bizuno_order_exported'))) { return; } // already downloaded, prevents duplicate download errors
        update_post_meta($order_id, 'bizuno_order_exported', 0);
        if ('yes' == strtolower(get_option('bizuno_api_autodownload', false))) {
            $this->bizuno_api_order_download($order_id); // call return to bit bucket as as all messsages are suppressed
//          $order = new WC_Order($order_id);
//          $order->update_status('completed'); // commented out as not completed until shipped
            $GLOBALS['bizunoExport'] = true; // turn off messaging since autodownload
        }
    }

    // Calls the API method processSend to send to Bizuno
    private function bizuno_api_order_download($order_id=0, $verbose=false)
    {
        require_once ('controllers/portal/controller.php');
        $_GET['bizRt'] = "bizuno/import/processRcv";
        $_GET['rID']   = !empty($order_id) ? $order_id : intval($_GET['biz_order_id']);
        $ctl = new \bizuno\portalCtl();
        $ctl->compose();
        $msgStack = $ctl->getMessages();
        if ($verbose) { $this->setNotices($msgStack); } // insert results into WordPress message system to dispaly after redirect
    }

    public function add_shipped_to_order_statuses( $order_statuses ) {
        $new_order_statuses = [];
        foreach ( $order_statuses as $key => $status ) {
            $new_order_statuses[ $key ] = $status;
            if ( 'wc-processing' === $key ) { $new_order_statuses['wc-shipped'] = 'Shipped'; }
        }
        return $new_order_statuses;
    }

    public function bizuno_api_order_column_header( $columns ) {
        $new_columns = [];
        foreach ( $columns as $column_name => $column_info ) {
            if ( 'order_date' === $column_name ) { $new_columns['bizuno_download'] = __( 'Exported', 'bizuno_api' ); }
            $new_columns[$column_name] = $column_info;
        }
        return $new_columns;
    }

    public function bizuno_api_order_column_content($column) {
        global $post;
        if ( 'bizuno_download' === $column ) {
            $order    = wc_get_order( $post->ID );
            $exported = $order->get_meta('bizuno_order_exported', true, 'edit');
            if (!$exported) {
                $tip = '';
                echo '<button type="button" class="order-status status-processing tips" data-tip="'.$tip.'">'.__( 'No', 'bizuno_api' ).'</button>';
            } else { echo '&nbsp;'; }
        }
    }

    // Add download button to Preview pop up
    public function bizuno_api_order_preview_filter( $data, $order ) {
        $data['bizuno_order_exported'] = $order->get_meta('bizuno_order_exported', true, 'edit') ? 'none' : 'block';
        return $data;
    }

    public function bizuno_api_order_preview_action( $order ) { // passed $order (not used)
        $url = admin_url( 'admin-ajax.php?action=bizuno_api_order_download' );
        echo '<span style="display:{{ data.bizuno_order_exported }}"><a class="button button-primary button-large" onClick="window.location = \''.$url.'&biz_order_id={{ data.data.id }}\';">'.__( 'Export order to Bizuno', 'bizuno_api' ).'</a></span>'."\n";
    }

    // Add download button to order edit page
    public function bizuno_api_add_order_meta_box_filter( $actions ) {
        global $post;
        if (get_post_meta( $post->ID, 'bizuno_order_exported', true ) ) { return $actions; }
        $actions['bizuno_export_action'] = __('Export order to Bizuno', 'bizuno_api');
        return $actions;
    }

    // Import and Order to Bizuno, called from pull down when editing order
    public function bizuno_api_process_order_meta_box_action( $order ) {
        $order_id = $order->get_id();
        $this->bizuno_api_manual_download(true);
    }

    private function setNotices($msgStack=[]) {
        $error = $warning = $success = "";
        if (empty($msgStack)) {
            return $this->bizuno_api_add_notice("Unexpected response from the Bizuno API: ".print_r($msgStack, true), 'error', true);
        }
        foreach ($msgStack as $key => $value) {
            switch ($key) {
                case 'error':   foreach ($value as $msg) { $error   .= $msg['text']."\n"; } break;
                case 'caution':
                case 'warning': foreach ($value as $msg) { $warning .= $msg['text']."\n"; } break;
                case 'success': foreach ($value as $msg) { $success .= $msg['text']."\n"; } break;
            }
        }
        $result = 'error';
        if (!empty($success)) { $result = 'success'; $this->bizuno_api_add_notice( $success, 'success', true ); }
        if (!empty($warning)) { $result = 'warning'; $this->bizuno_api_add_notice( $warning, 'warning', true ); }
        if (!empty($error))   { $result = 'error';   $this->bizuno_api_add_notice( $error,   'error',   true ); }
        return $result;
    }

    public function bizuno_api_add_notice( $notice = "", $type = "warning", $dismissible = true ) {
        if (!empty($GLOBALS['bizunoExport'])) { return; }
        $notices = get_option( "bizuno_flash_notices", [] );
        $dismissible_text = ($dismissible) ? "is-dismissible" : "";
        array_push( $notices, ['notice'=>$notice, 'type'=>$type, 'dismissible'=>$dismissible_text] );
        update_option("bizuno_flash_notices", $notices );
    }

    public function bizuno_api_display_notices() {
        $notices = get_option( "bizuno_flash_notices", [] );
        foreach ( $notices as $notice ) {
            printf('<div class="notice notice-%1$s %2$s"><p>%3$s</p></div>', $notice['type'], $notice['dismissible'], $notice['notice'] );
        }
        if (!empty( $notices)) { delete_option( "bizuno_flash_notices", [] ); }
    }

    public function bizuno_api_add_section( $sections ) {
        $sections['bizuno_api'] = __( 'Bizuno API', 'text-domain' );
        return $sections;
    }

    public function bizuno_api_all_setting( $settings, $current_section ) {
        if ( $current_section <> 'bizuno_api' ) { return $settings; }
        unset($settings);
        $settings[] = ['name'=> __( 'Bizuno API', 'text-domain' ),          'id'=>'bizuno_api',            'type'=>'title',
            'desc'    => __( 'The following options are used to configure the interface to Bizuno Accounting', 'text-domain' ) ];
        $settings[] = ['name'=> __( 'Bizuno Order Prefix', 'bizuno_api'),   'id'=>'bizuno_api_prefix',     'type'=>'text',
            'desc_tip'=> __( 'Leave blank for no prefix.', 'bizuno_api'),
            'desc'    => __( 'Placing a value here will help identify where the orders originated from.', 'bizuno_api') ];
        $settings[] = ['name'=> __( 'Bizuno Customer Prefix', 'bizuno_api'),'id'=>'bizuno_api_prefix_cust','type'=>'text',
            'desc_tip'=> __( 'Leave blank for no prefix.', 'bizuno_api'),
            'desc'    => __( 'Placing a value here will help identify where your customers originated from.', 'bizuno_api') ];
        $settings[] = ['name'=> __( 'Ship Complete Status', 'bizuno_api' ), 'id'=>'bizuno_ship_status',    'type'=>'select','options'=>wc_get_order_statuses(),'default'=>'wc-completed',
            'desc_tip'=> __( 'Default WooCommerce value is wc-completed.', 'bizuno_api'),
            'desc'    => __( 'If not set, the default WooCommerce value will be used.', 'bizuno_api') ];
        $settings[] = ['name'=> __( 'Automatically Download Orders', 'bizuno_api' ), 'id'=>'bizuno_api_autodownload', 'type'=>'checkbox',
            'desc'    => __( 'If checked, your orders will automatically be downloaded to Bizuno just after the customer completes the order.', 'bizuno_api') ];
        $settings[] = ['type'=>'sectionend', 'id'=>'bizuno_api'];
        return $settings;
    }

    public function daily_cron()
    {
        require_once ("controllers/portal/controller.php");
        require_once (BIZBOOKS_ROOT ."controllers/phreebooks/functions.php");
        \bizuno\periodAutoUpdate(false);
    }

    public static function activate()
    {
        global $wpdb;
        require_once('bizunoCFG.php');
/*      $oPage = get_page_by_path('bizuno-office'); // need to create page placeholder for office
        if (empty($oPage)) {
            $my_post = ['post_title'=>'Bizuno Office', 'post_status'=>'publish', 'post_type'=>'page',
                'post_content'  => '<p>This is a placeholder page to display Bizuno Office on your site. Any content here will not be displayed.'];
            wp_insert_post( $my_post );
        }
        update_user_meta( get_current_user_id(), 'bizoffice_enable', 1 ); */
        /********* Accounting *********/
        $aPage = get_page_by_path('bizuno'); // need to create page placeholder for accounting
        if (empty($aPage)) {
            $my_post = ['post_title'=>'Bizuno', 'post_status'=>'publish', 'post_type'=>'page', 'post_name'=>'bizuno',
                'post_content'=>"This page is reserved for authorized users of Bizuno Accounting/ERP.
To access Bizuno, please <a href=\"/wp-login.php\">click here</a> to log into your WordPress site and select Bizuno Accounting from the setting menu in the upper right corner of the screen.
If Bizuno Accounting is not an option, see your administrator to gain permission.</p>
<p>Administrators: To authorize a user, navigate to the WordPress administration page -> Users -> search  username/eMail.
Edit the user and check the \'Allow access to: <My Business>\' box along with a role and click Save.</p>"];
            wp_insert_post( $my_post );
        }
        foreach ($GLOBALS['bizPortal'] as $biz) { update_user_meta( get_current_user_id(), 'bizbooks_enable_'.$biz['id'], 1 ); }
        if (!wp_next_scheduled('bizuno_daily_event')) { wp_schedule_event(time(), 'daily', 'bizuno_daily_event'); }
        /********* API *********/
        if ( is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
            // set all existing orders to downloaded to hide Download button for past orders
            $orders = $wpdb->get_results( "SELECT `ID` FROM `{$wpdb->prefix}posts` WHERE `post_type` LIKE 'shop_order'", ARRAY_A);
            foreach ($orders as $order) { update_post_meta($order['ID'], 'bizuno_order_exported', 1); }
        }
    }

    public static function deactivate()
    {
        /********* Accounting *********/
        delete_option('bizuno_active', false);
        wp_clear_scheduled_hook('bizuno_daily_event');
        /********* API *********/
        update_option('bizuno_api_active', false);
    }
    public function bizuno_debug_log_write() {
        if (!empty($GLOBALS['bizDebug'])) {
            require_once ('controllers/portal/controller.php');
            new \bizuno\portalCtl();
            \bizuno\msgTrap();
            \bizuno\msgDebug("\nCaptured from WordPress Plugin: \n".print_r(implode("\n",$GLOBALS['bizDebug']), true));
            \bizuno\msgDebugWrite();
            unset($GLOBALS['bizDebug']);
        }
    }
    /**************** Private Mdethods *****************/
    private function validateUser()
    {
        $access = false;
        if (!empty($_GET['bizRt']) && 'myPortal'==substr($_GET['bizRt'], 0, 8)) { return true; }
        if ( !is_user_logged_in() ) { return false; }
        $userID = get_current_user_id();
        foreach ($GLOBALS['bizPortal'] as $biz) {
            if ( !empty(get_user_meta( $userID, 'bizbooks_enable_'.$biz['id'], true) ) ) { $access = true; break; }
        }
        return $access;
    }
}
new bizuno_accounting();

/**
 * Uninstall should remove all documents and settings, essentially a clean wipe.
 * WARNING: Needs to be outside of class or WordPress errors
 */
function bizuno_uninstall() {
    global $wpdb;
    $upload_dir = wp_upload_dir();
    $options = ['bizuno_active','bizuno_api_user','bizuno_api_pw','bizuno_api_url','bizuno_api_prefix','bizuno_api_prefix_cust','bizuno_api_autodownload','bizuno_api_active'];
    foreach ($options as $option) { delete_option($option); delete_site_option($option); }
    /********* Office *********/
//  $result = $wpdb->get_results("SELECT user_id FROM `".$wpdb->perfix."usermeta` WHERE meta_key='bizoffice_enable'");
//  if (!empty($result)) { foreach ($result as $user) { delete_user_meta( $user->user_id, 'bizoffice_enable' ); } }
    /********* Accounting *********/
    $tables = $wpdb->get_results("SHOW TABLES LIKE '{$wpdb->prefix}bizuno_%'", ARRAY_A);
    foreach ($tables as $row) { $table = array_shift($row); $wpdb->query("DROP TABLE IF EXISTS $table"); } // drop the Bizuno tables
    bizunoRmdir($upload_dir['basedir'].'/bizuno');
    /********* API *********/
    $wpdb->get_results("DELETE FROM `{$wpdb->prefix}postmeta` WHERE meta_key='bizuno_order_exported'");
}

/**************************************************************************************/
//                  Support functions
/**************************************************************************************/
//Recursive support function to remove Bizuno files from the uploads folder
function bizunoRmdir($dir) {
    if (!is_dir($dir)) { return; }
    $objects = scandir($dir);
    foreach ($objects as $object) {
        if ($object == "." || $object == "..") { continue; }
        if (is_dir($dir."/".$object)) { bizunoRmdir($dir."/".$object); } else { unlink($dir."/".$object);  }
    }
    rmdir($dir);
}

/**
 * DEPRECATED - Function call commented out
 */
function bizuno_options_page()
{
    echo '<div><h1>Bizuno Settings</h1><form method="post" action="options.php">'.settings_fields( 'bizuno_options_group' );
    echo '<h3>Go PRO! Purchase the Bizuno-Pro plugin from PhreeSoft and max out your business system.</h3>';
    echo '<p><a href="https://www.phreesoft.com">GO PRO!</a></p>';
    echo '<table><tr valign="top"><th scope="row"><label for="bizuno_pro_key">Pro Key</label></th>';
    echo '<td><input type="text" id="bizuno_pro_key" name="bizuno_pro_key" value="'.get_option('bizuno_pro_key').'" /></td></tr></table>';
    submit_button();
    echo '</form></div>';
}

function bizuno_html() {
    echo "<p>Please Note: This access method to Bizuno Accounting has deprecated!</p>";
    echo "<p>Bizuno Accounting now only runs in full screen mode to allow all users to use the app in a consistent environment.</p>";
    echo "<p>Permission must be granted by checking the 'Enable Bizuno Accounting' checkbox in each individual users profile. Once access has been granted, Bizuno Accounting can be accessed from the Users profile drop down menu after the user has logged into your site.</p>";
}
